#property indicator_separate_window
#property indicator_buffers 1
#property indicator_color1 Red

double Fisher[];
double stoch[];
double Value1[];
double f[];

//  : 0 - Close, 1 - Open, 2 - High, 3 - Low, 4 - Median, 5 - Typical, 6 - Weighted
extern int PriceMode = PRICE_CLOSE;
//  
extern int StochPeriod = 21;
//   : 0 - Price/Price, 1 - High/Low
int StochMode = 0;
//  1- 
// 0 - Hull type MA
// 1 - HMA
// 2 - LWMA
// 3 - EMA
int SmoothType1 = 3;
//  1- 
int SmoothPeriod1 = 4;
// ,    
// 1.0 - 
// 0.5 - 
double k = 1;
//  2- 
// 0 - Hull type MA
// 1 - HMA
// 2 - LWMA
// 3 - EMA
int SmoothType2 = 3;
//  2- 
int SmoothPeriod2 = 4;

int buffers = 0;
int drawBegin = 0;

int init() 
{
   if (StochPeriod < 3) 
      StochPeriod = 3;
   drawBegin = StochPeriod;      
   initBuffer(Fisher, "Fisher", DRAW_LINE);
   initBuffer(Value1);
   initBuffer(stoch);
   initBuffer(f);
   IndicatorBuffers(buffers);
   string name = "FT("+PriceMode+","+StochPeriod+","+StochMode+","
      +SmoothType1+","+SmoothPeriod1+","
      +DoubleToStr(k,2)+","+SmoothType2+","+SmoothPeriod2+")";
   IndicatorShortName(name);
   SetIndexLabel(0,name);
   return (0);
}
  
int start() 
{
   if (Bars <= StochPeriod)  
      return (0);
   int countedBars = IndicatorCounted();
   if (countedBars < 0) 
      return (-1);
   if (countedBars > 0) 
      countedBars--;
   int s, limit = Bars - countedBars - 1;
   for (s = limit; s >= 0; s--) 
   {
      double price = P(s);
      double MaxH = price;
      double MinL = price;
      for (int i = 0; i < StochPeriod; i++) 
      {
         if (StochMode == 0)
         {
         price = P(s + i);
         if (price > MaxH) 
            MaxH = price;
         if (price < MinL) 
            MinL = price;
         }
         else
         {
            MaxH = MathMax(MaxH,High[s+i]);
            MinL = MathMin(MinL,Low[s+i]);
         }
      }
      stoch[s] = 2.0 * ((P(s) - MinL) / (MaxH - MinL) - 0.5);
   }
   smooth(stoch,Value1,SmoothType1,SmoothPeriod1,limit);
   for (s = limit; s >= 0; s--)
   {
      f[s] = MathLog((1.0 + val(Value1[s])) / (1.0 - val(Value1[s]))) / 2.0;
   }    
   smooth(f,Fisher,SmoothType2,SmoothPeriod2,limit);
   return (0);
}

double P(int index) 
{
   return (iMA(NULL,0,1,0,MODE_SMA,PriceMode,index));
}

void initBuffer(double array[], string label = "", 
   int type = DRAW_NONE, int arrow = 0, int style = EMPTY, 
   int width = EMPTY, color clr = CLR_NONE) 
{
   SetIndexBuffer(buffers, array);
   SetIndexLabel(buffers, label);
   SetIndexEmptyValue(buffers, EMPTY_VALUE);
   SetIndexDrawBegin(buffers, drawBegin);
   SetIndexShift(buffers, 0);
   SetIndexStyle(buffers, type, style, width);
   SetIndexArrow(buffers, arrow);
   buffers++;
}

double val(double v)
{
   return (MathMax(MathMin(k * v,0.999),-0.999));
}

void smooth(double inp[], double& out[], int type, int len, int limit)
{
   switch(type)
   {
      case 0:
         HTMA(Bars-StochPeriod,len,inp,out);
         break;
      case 1:
         HMA(len,inp,out);
         break;
      case 2:
         LWMA(len,inp,out);
         break;
      case 3:
         EMA(len,inp,out);
         break;
   }
}

void HTMA(int N,int HMALen,double input[],double &output[])
{
   double ma1,ma2,hma,mix,mixprime;
   ma1=input[N-1];
   ma2=ma1;
   mix=3.0/(2.0 + HMALen);
   mixprime=1.0-mix;
   for(int i=N-2;i>=0;i--)
   {
      ma1=mixprime*ma1 + mix*input[i];
      ma2=mixprime*ma2 + mix*ma1;
      output[i]=3.0*ma1-2.0*ma2;
   }
}

void HMA(int per, double array[], double& out[])
{
   double tmp[];
   ArrayResize(tmp,Bars);
   ArraySetAsSeries(tmp,true);
   for(int i = Bars-1; i >= 0; i--)
   {
      tmp[i] =
         2*iMAOnArray(array,0,per/2,0,MODE_LWMA,i)
         -iMAOnArray(array,0,per,0,MODE_LWMA,i);
   }
   for(i = Bars-1; i >= 0; i--)
   {
      out[i] = iMAOnArray(tmp,0,MathSqrt(per),0,MODE_LWMA,i);
   }
}

void LWMA(int len, double inp[], double& out[])
{
   for(int i = Bars-1; i >= 0; i--)
   {
      out[i] = iMAOnArray(inp,0,len,0,MODE_LWMA,i);
   }
}

void EMA(int len, double inp[], double& out[])
{
   for(int i = Bars-1; i >= 0; i--)
   {
      out[i] = iMAOnArray(inp,0,len,0,MODE_EMA,i);
   }
}

